import os
# Assuming other necessary imports like json if used in missing functions
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QHBoxLayout, QGridLayout, QGroupBox, QScrollArea, QLineEdit, QComboBox, QPushButton, QCheckBox, QTextEdit, QListWidget, QFileDialog, QMessageBox, QLabel
# Assume missing functions like get_dev_status_js, get_file_associations, get_all_choice_tags, list_all_directories, check_all_files, get_readme, create_classifiers, get_installed_versions, scan_folder_for_required_modules, write_to_file, get_fun, upload_main, return_opposite_bool, licenses, create_setup_cfg, create_toml, create_setup, create_init, get_main_py, parse_setup are defined elsewhere and remain the same.

class MainWindow(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("create module")
        self.resize(800, 600)
        main_layout = QHBoxLayout()
        self.setLayout(main_layout)
        self.widgets = {}
        self.folder_bridge = {"MODULE_NAV_HISTORY": [], "MODULE_FOLDER": None}
        descriptions = self.get_descriptions_layout()
        main_layout.addWidget(descriptions)
        top_layout = QVBoxLayout()
        self.listbox = QListWidget()
        self.listbox.addItems(os.listdir(os.getcwd()))
        self.listbox.setObjectName("-DIRECTORY_LIST-")
        self.listbox.itemClicked.connect(self.on_directory_selected)
        self.listbox.setFixedWidth(200)
        self.listbox.setFixedHeight(300)
        back_btn = QPushButton("<-")
        back_btn.setObjectName("<-")
        forward_btn = QPushButton("->")
        forward_btn.setObjectName("->")
        buttons_layout = QHBoxLayout()
        buttons_layout.addWidget(back_btn)
        buttons_layout.addWidget(forward_btn)
        listbox_layout = QVBoxLayout()
        listbox_layout.addWidget(self.listbox)
        listbox_layout.addLayout(buttons_layout)
        listbox_column = QWidget()
        listbox_column.setLayout(listbox_layout)
        check_marks = self.create_check_marks()
        choice_tag = self.create_choice_tag_layout()
        dev_status = self.create_dev_status()
        status_layout = QVBoxLayout()
        status_layout.addWidget(check_marks)
        status_layout.addWidget(choice_tag)
        status_layout.addWidget(dev_status)
        status_column = QWidget()
        status_column.setLayout(status_layout)
        top_row = QHBoxLayout()
        top_row.addWidget(listbox_column)
        top_row.addWidget(status_column)
        top_widget = QWidget()
        top_widget.setLayout(top_row)
        top_layout.addWidget(top_widget)
        module_browser = self.get_module_folder_browser()
        top_layout.addWidget(module_browser)
        upload_btn = QPushButton("dev_status")
        upload_btn.setObjectName("-UPLOAD_MODULE-")
        top_layout.addWidget(upload_btn)
        top_frame = QGroupBox("")
        top_frame.setLayout(top_layout)
        main_layout.addWidget(top_frame)
        self.collect_widgets(self)
        for key, widget in self.widgets.items():
            if isinstance(widget, QPushButton):
                widget.clicked.connect(self.on_button_clicked)
            if isinstance(widget, QCheckBox):
                widget.stateChanged.connect(self.on_checkbox_changed)
            if isinstance(widget, QLineEdit):
                widget.textChanged.connect(self.on_input_changed)
            if isinstance(widget, QComboBox):
                widget.currentIndexChanged.connect(self.on_combo_changed)

    def collect_widgets(self, widget):
        for child in widget.children():
            if child.objectName():
                self.widgets[child.objectName()] = child
            self.collect_widgets(child)

    def get_values(self):
        try:
            values = {}
            for key, widget in self.widgets.items():
                if isinstance(widget, QLineEdit):
                    values[key] = widget.text()
                elif isinstance(widget, QComboBox):
                    values[key] = widget.currentText()
                elif isinstance(widget, QCheckBox):
                    values[key] = widget.isChecked()
                elif isinstance(widget, QTextEdit):
                    values[key] = widget.toPlainText()
                elif isinstance(widget, QListWidget):
                    values[key] = [item.text() for item in widget.selectedItems()]
            return values
        except:
            return {}

    def update_values(self, key, args):
        if key in self.widgets:
            widget = self.widgets[key]
            if "value" in args:
                if isinstance(widget, QLineEdit):
                    widget.setText(str(args["value"]))
                elif isinstance(widget, QComboBox):
                    widget.setCurrentText(str(args["value"]))
                elif isinstance(widget, QCheckBox):
                    widget.setChecked(args["value"])
                elif isinstance(widget, QTextEdit):
                    widget.setPlainText(str(args["value"]))
                elif isinstance(widget, QListWidget):
                    widget.clear()
                    widget.addItems(args["values"])
            if "disabled" in args:
                widget.setEnabled(not args["disabled"])

    def browse_folder(self, input_widget):
        directory = QFileDialog.getExistingDirectory(self, "Select Folder", input_widget.text())
        if directory:
            input_widget.setText(directory)

    def browse_file(self, input_widget):
        file = QFileDialog.getOpenFileName(self, "Select File", input_widget.text())[0]
        if file:
            input_widget.setText(file)

    def create_dev_status(self):
        layout = QGridLayout()
        dev_status_json = get_dev_status_js()
        row, col = 0, 0
        for each in dev_status_json.keys():
            dev_status_value = dev_status_json[each]
            lay = QHBoxLayout()
            if each == "Programming Language":
                combo = QComboBox()
                combo.addItems(dev_status_value)
                combo.setCurrentText(dev_status_value[0])
                combo.setObjectName("-DEV_STATUS_" + each + '-')
                combo.setFixedSize(90, 20)
                lay.addWidget(combo)
                input_v = QLineEdit("version")
                input_v.setObjectName("-VERSION_PY_" + each + "-")
                input_v.setFixedSize(40, 20)
                lay.addWidget(input_v)
            else:
                combo = QComboBox()
                combo.addItems(dev_status_value)
                combo.setCurrentText(dev_status_value[0])
                combo.setObjectName("-DEV_STATUS_" + each + '-')
                combo.setFixedSize(150, 20)
                lay.addWidget(combo)
            btn = QPushButton("+")
            btn.setObjectName("-DEV_STATUS_" + each + "_BUTTON-")
            lay.addWidget(btn)
            frame = QGroupBox(each)
            frame.setObjectName("-COLUMN_" + each + "-")
            frame.setLayout(lay)
            layout.addWidget(frame, row, col)
            col += 1
            if col == 3:
                col = 0
                row += 1
        outer_frame = QGroupBox("Choose Dev Status")
        outer_frame.setLayout(layout)
        scroll = QScrollArea()
        scroll.setWidget(outer_frame)
        scroll.setWidgetResizable(True)
        scroll.setFixedSize(200, 200)
        return scroll

    def create_check_marks(self):
        file_associations = get_file_associations()
        check_list = file_associations.keys()
        layout = QGridLayout()
        row, col = 0, 0
        for i, each in enumerate(check_list):
            check = QCheckBox(file_associations[each]["file"])
            check.setChecked(True)
            check.setObjectName(each[:-1] + '_CHECK-')
            layout.addWidget(check, row, col)
            col += 1
            if col == 2:
                col = 0
                row += 1
        create_btn = QPushButton("Create Files")
        create_btn.setObjectName("Create Files")
        layout.addWidget(create_btn, row, 0)
        create_module_btn = QPushButton("Create Module")
        create_module_btn.setObjectName("-CREATE_FOLDER-")
        layout.addWidget(create_module_btn, row, 1)
        frame = QGroupBox("File Checklist")
        frame.setLayout(layout)
        return frame

    def create_inputs(self, string):
        lay = QHBoxLayout()
        lay.addWidget(QLabel(string + ':'))
        if string == "long_description":
            le = QLineEdit(os.getcwd())
            le.setObjectName(string)
            lay.addWidget(le)
            btn = QPushButton("Browse")
            btn.clicked.connect(lambda: self.browse_file(le))
            lay.addWidget(btn)
        elif string == "version":
            for j in range(4):
                le = QLineEdit("0")
                le.setObjectName(string + '_' + str(j))
                le.setFixedSize(20, 20)
                lay.addWidget(le)
                if j < 3:
                    lay.addWidget(QLabel('.'))
        elif string == "description":
            ml = QTextEdit()
            ml.setObjectName(string)
            lay.addWidget(ml)
        else:
            le = QLineEdit()
            le.setObjectName(string)
            le.setFixedSize(200, 20)
            lay.addWidget(le)
        widget = QWidget()
        widget.setLayout(lay)
        return widget

    def create_choice_tag_layout(self):
        lay = QVBoxLayout()
        choice_tags = get_all_choice_tags()
        for each in choice_tags:
            if each not in ["description", "long_description"]:
                lay.addWidget(self.create_inputs(each))
        frame = QGroupBox("Choice Tags")
        frame.setLayout(lay)
        return frame

    def get_descriptions_layout(self):
        lay = QVBoxLayout()
        description_list = ["description", "long_description"]
        for each in description_list:
            lay.addWidget(self.create_inputs(each))
        frame = QGroupBox("Descriptions")
        frame.setLayout(lay)
        return frame

    def get_module_folder_browser(self, initial_folder="/home/john-putkey/Documents/modules/abstract_audio"):
        frame_lay = QHBoxLayout()
        permenant = QLineEdit(initial_folder)
        permenant.setObjectName("-MODULE_PERMENANT_PATH-")
        frame_lay.addWidget(permenant)
        folder = QLineEdit(initial_folder)
        folder.setObjectName("-MODULE_FOLDER_PATH-")
        frame_lay.addWidget(folder)
        browse = QPushButton("Browse")
        browse.setObjectName("-MODULE_FOLDER_BROWSER-")
        frame_lay.addWidget(browse)
        check = QCheckBox("choose folder")
        check.setChecked(False)
        check.setObjectName("-LOCK_CHECKBOX-")
        frame_lay.addWidget(check)
        frame = QGroupBox("Choose Module Directory")
        frame.setLayout(frame_lay)
        return frame

    def new_layout(self, key, i):
        dev_status_json = get_dev_status_js()
        value = dev_status_json[key]
        lay = QHBoxLayout()
        combo = QComboBox()
        combo.setObjectName("-DEV_STATUS_" + key + str(i) + '-')
        combo.addItems(value)
        combo.setCurrentText(value[0])
        lay.addWidget(combo)
        if key == "Programming Language":
            input_v = QLineEdit("version")
            input_v.setObjectName("-VERSION_PY_" + key + str(i) + "-")
            lay.addWidget(input_v)
        widget = QWidget()
        widget.setLayout(lay)
        return widget

    def on_button_clicked(self):
        sender = self.sender()
        event = sender.objectName()
        values = self.get_values()
        if event == "-UPLOAD_MODULE-":
            upload_main(project_dir=values["-MODULE_PERMENANT_PATH-"])
        elif event == "Create Files":
            self.on_create_files()
        elif event == "-CREATE_FOLDER-":
            self.on_create_folder()
        elif event == "-MODULE_FOLDER_BROWSER-":
            self.browse_folder(self.widgets["-MODULE_FOLDER_PATH-"])
        elif "-DEV_STATUS_" in event and event.endswith("_BUTTON-"):
            key = event[len("-DEV_STATUS_") : -len("_BUTTON-")]
            self.on_add_dev_status(key)

    def on_create_files(self):
        values = self.get_values()
        args = {"readme_path": values["long_description"],
                'README.md': values["long_description"],
                "license": values["-DEV_STATUS_License-"],
                "package_name": values["package_name"],
                "version": values["version_0"] + '.' + values["version_1"] + '.' + values["version_2"] + '.' + values["version_3"],
                "author": values["author"],
                "author_email": values["author_email"],
                "description": values["description"],
                "url": values["url"],
                "classifiers": create_classifiers(),
                "install_requires": get_installed_versions(scan_folder_for_required_modules(values["-MODULE_PERMENANT_PATH-"], "src"))[0]}
        file_associations = get_file_associations()
        for each in file_associations.keys():
            if values[each[:-1] + '_CHECK-']:
                contents = get_fun({"args": args, "global": globals(), "name": file_associations[each]["function"]})
                if contents is not None:
                    file_path = os.path.join(values["-MODULE_PERMENANT_PATH-"], file_associations[each]["directory"], file_associations[each]["file"])
                    write_to_file(filepath=file_path, contents=contents)

    def on_create_folder(self):
        dialog = QDialog(self)
        dialog.setWindowTitle("please choose a folder name and path")
        lay = QVBoxLayout()
        lay.addWidget(QLabel('Enter the new folder name:'))
        new_folder_name = QLineEdit()
        lay.addWidget(new_folder_name)
        lay.addWidget(QLabel('Select Parent Directory:'))
        parent_dir = QLineEdit(self.get_values()["-MODULE_PERMENANT_PATH-"])
        lay.addWidget(parent_dir)
        browse = QPushButton("Browse")
        browse.clicked.connect(lambda: self.browse_folder(parent_dir))
        lay.addWidget(browse)
        buttons = QHBoxLayout()
        create = QPushButton("Create")
        cancel = QPushButton("Cancel")
        buttons.addWidget(create)
        buttons.addWidget(cancel)
        lay.addLayout(buttons)
        dialog.setLayout(lay)
        create.clicked.connect(dialog.accept)
        cancel.clicked.connect(dialog.reject)
        if dialog.exec() == QDialog.Accepted:
            values = {"-NEW_FOLDER_NAME-": new_folder_name.text(), "-PARENT_DIR-": parent_dir.text()}
            if values['-NEW_FOLDER_NAME-'] and values['-PARENT_DIR-']:
                folder_path = os.path.join(values['-PARENT_DIR-'], values['-NEW_FOLDER_NAME-'])
                os.makedirs(folder_path, exist_ok=True)
                self.update_values("-MODULE_FOLDER_PATH-", {"value": folder_path})
                self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(folder_path)})

    def on_add_dev_status(self, key):
        i = 0
        while "-DEV_STATUS_" + key + str(i) + '-' in self.widgets:
            i += 1
        new = self.new_layout(key, i)
        frame = self.widgets["-COLUMN_" + key + "-"]
        frame.layout().addWidget(new)
        for child in new.findChildren(QWidget):
            if child.objectName():
                self.widgets[child.objectName()] = child
                if isinstance(child, QComboBox):
                    child.currentIndexChanged.connect(self.on_combo_changed)
                # Add other connections if needed

    def on_checkbox_changed(self, state):
        sender = self.sender()
        event = sender.objectName()
        if event == "-LOCK_CHECKBOX-":
            values = self.get_values()
            if not values["-LOCK_CHECKBOX-"]:
                self.update_values("-MODULE_PERMENANT_PATH-", {"disabled": False})
                self.folder_bridge["MODULE_FOLDER"] = None
            else:
                self.update_values("-MODULE_PERMENANT_PATH-", {"value": values["-MODULE_FOLDER_PATH-"]})
                self.update_values("-MODULE_PERMENANT_PATH-", {"disabled": True})
                self.folder_bridge["MODULE_FOLDER"] = values["-MODULE_PERMENANT_PATH-"]
                check_all_files(self.folder_bridge["MODULE_FOLDER"])

    def on_input_changed(self):
        sender = self.sender()
        event = sender.objectName()
        if event == "-MODULE_FOLDER_PATH-":
            values = self.get_values()
            if os.path.exists(values["-MODULE_FOLDER_PATH-"]):
                self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(values["-MODULE_FOLDER_PATH-"])})
                folder = values["-MODULE_FOLDER_PATH-"]
                if folder:
                    self.update_values("-MODULE_FOLDER_PATH-", {"value": folder})
                    if not values["-LOCK_CHECKBOX-"]:
                        self.update_values("-MODULE_PERMENANT_PATH-", {"value": folder})

    def on_combo_changed(self, index):
        sender = self.sender()
        event = sender.objectName()
        # Add handling if needed for specific combos

    def on_directory_selected(self, item):
        values = self.get_values()
        chosen_folder = item.text()
        chosen_path = os.path.join(values["-MODULE_FOLDER_PATH-"], chosen_folder) if values["-MODULE_FOLDER_PATH-"] else None
        if chosen_path and os.path.isdir(chosen_path):
            self.update_values("-MODULE_FOLDER_PATH-", {"value": chosen_path})
            self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(chosen_path)})
            if not values["-LOCK_CHECKBOX-"]:
                self.update_values("-MODULE_PERMENANT_PATH-", {"value": chosen_path})
            self.folder_bridge["MODULE_FOLDER"] = chosen_path
        else:
            QMessageBox.warning(self, "warning", f"{chosen_folder} is not a directory.")

    def on_back(self):
        values = self.get_values()
        current_folder = values["-MODULE_FOLDER_PATH-"]
        parent_folder = os.path.dirname(current_folder) if current_folder else None
        if parent_folder:
            if self.folder_bridge["MODULE_FOLDER"] is not None:
                all_dirs = list_all_directories(self.folder_bridge["MODULE_FOLDER"])
                if parent_folder in all_dirs:
                    self.update_values("-MODULE_FOLDER_PATH-", {"value": parent_folder})
                    self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(parent_folder)})
            elif os.path.exists(parent_folder):
                self.update_values("-MODULE_FOLDER_PATH-", {"value": parent_folder})
                self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(parent_folder)})
                self.folder_bridge["MODULE_NAV_HISTORY"].append(current_folder)

    def on_forward(self):
        if self.folder_bridge["MODULE_NAV_HISTORY"]:
            next_folder = self.folder_bridge["MODULE_NAV_HISTORY"].pop()
            if self.folder_bridge["MODULE_FOLDER"] is not None:
                all_dirs = list_all_directories(self.folder_bridge["MODULE_FOLDER"])
                if next_folder in all_dirs:
                    self.update_values("-MODULE_FOLDER_PATH-", {"value": next_folder})
                    self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(next_folder)})
            elif os.path.exists(next_folder):
                self.update_values("-MODULE_FOLDER_PATH-", {"value": next_folder})
                self.update_values("-DIRECTORY_LIST-", {"values": os.listdir(next_folder)})

# The rest of the functions remain the same, as they do not involve GUI elements.

# To run the application
if __name__ == "__main__":
    app = QApplication([])
    window = MainWindow()
    window.show()
    app.exec()



from ..functions import *
# — UI helpers —
def browse_file(self):
    f = QFileDialog.getOpenFileName(self, "Select File")[0]
    if f:
        self.file_in.setText(f)
def preview_patch(self):
    file_path = self.file_in.text().strip()
    if not file_path or not os.path.isfile(file_path):
        QMessageBox.critical(self, "Error", "Invalid file.")
        return
    diff = self.diff_text.toPlainText().strip()
    if not diff:
        QMessageBox.critical(self, "Error", "No diff provided.")
        return
    try:
        with open(file_path, 'r') as f:
            original = f.read()
        patched = apply_custom_diff(original.splitlines(), diff.splitlines())
        self.preview.setPlainText(patched)
    except ValueError as e:
        QMessageBox.critical(self, "Error", str(e))
def save_patch(self):
    file_path = self.file_in.text().strip()
    patched = self.preview.toPlainText()
    if patched:
        with open(file_path, 'w') as f:
            f.write(patched + '\n')
        QMessageBox.information(self, "Success", "File saved.")
    else:
        QMessageBox.warning(self, "Warning", "No preview to save.")





from ...imports import *
from ....functions import *
def initializeInit(self):
    self.fn_filter_mode = "io"   # "source" | "io" | "all"
    self.current_fn = None       # last clicked function name

    self.setWindowTitle("🔍 React Build Finder")
    self.resize(1100, 720)

    self.cb_try_alt_ext = QCheckBox("Try alternate extensions")
    self.cb_try_alt_ext.setChecked(True)

    # state
    self.last_output = ""
    self.last_errors_only = ""
    self.last_warnings_only = ""

    # inputs
    try:
        default_user = os.getlogin()
    except Exception:
        default_user = "solcatcher"
    self.user_in = QLineEdit(default_user)
    self.user_in.setPlaceholderText("ssh solcatcher")
    self.path_in = QLineEdit(os.getcwd())
    self.path_in.setPlaceholderText("Project path (folder with package.json)")

    # buttons
    self.run_btn = QPushButton("Run");    self.run_btn.clicked.connect(self.start_work)
    self.rerun_btn = QPushButton("Re-run build"); self.rerun_btn.clicked.connect(self.start_work)
    self.clear_btn = QPushButton("Clear"); self.clear_btn.clicked.connect(self.clear_ui)

    # log + filter
    self.log_view = QTextEdit(); self.log_view.setReadOnly(True); self.log_view.setLineWrapMode(QTextEdit.NoWrap)
    self.filter_group, rbs = self.create_radio_group(["All", "Errors only", "Warnings only"], 0, self.apply_log_filter)
    self.rb_all, self.rb_err, self.rb_wrn = rbs

    # lists
    self.errors_list  =  build_warnings_list(self)
    self.warnings_list =  build_warnings_list(self)



from .runner import *
from .functions_page import *
from .initialize_init import *
from .finder import *
from .apigui import *
def getManagerFuncs(self):
    self.getRunner = getRunner
    self.initializeInit = initializeInit



# functions_console.py
from PyQt5.QtCore import Qt, pyqtSignal, QPoint, QRect, QSize
from PyQt5.QtGui import QTextCursor
from PyQt5.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QScrollArea, QLineEdit, QPushButton,
    QListWidget, QButtonGroup, QRadioButton, QComboBox, QLabel, QTextEdit,
    QSizePolicy, QLayout
)
import os

# --- FlowLayout (chips that wrap) -------------------------------------------
class FlowLayout(QLayout):
    def __init__(self, parent=None, margin=0, hspacing=8, vspacing=6):
        super().__init__(parent)
        self._items = []; self._h = hspacing; self._v = vspacing
        self.setContentsMargins(margin, margin, margin, margin)
    def addItem(self, item): self._items.append(item)
    def addWidget(self, w): super().addWidget(w)
    def count(self): return len(self._items)
    def itemAt(self, i): return self._items[i] if 0 <= i < len(self._items) else None
    def takeAt(self, i): return self._items.pop(i) if 0 <= i < len(self._items) else None
    def expandingDirections(self): return Qt.Orientations(0)
    def hasHeightForWidth(self): return True
    def heightForWidth(self, w): return self._doLayout(QRect(0,0,w,0), True)
    def setGeometry(self, r): super().setGeometry(r); self._doLayout(r, False)
    def sizeHint(self): return self.minimumSize()
    def minimumSize(self):
        s = QSize()
        for it in self._items: s = s.expandedTo(it.minimumSize())
        m = self.contentsMargins()
        s += QSize(m.left()+m.right(), m.top()+m.bottom())
        return s
    def _doLayout(self, rect, test):
        x = rect.x(); y = rect.y(); lineH = 0
        m = self.contentsMargins()
        x += m.left(); y += m.top(); right = rect.right() - m.right()
        for it in self._items:
            sz = it.sizeHint(); w = sz.width(); h = sz.height()
            if x + w > right and lineH > 0:
                x = rect.x() + m.left(); y += lineH + self._v; lineH = 0
            if not test: it.setGeometry(QRect(QPoint(x, y), sz))
            x += w + self._h; lineH = max(lineH, h)
        return y + lineH + m.bottom() - rect.y()

# --- Console ---------------------------------------------------------------
class FunctionConsole(QWidget):
    functionSelected = pyqtSignal(str)
    scanRequested = pyqtSignal(str)  # scope string ("all" | "reachable")

    def __init__(self, parent=None, use_flow=True):
        super().__init__(parent)
        self.func_map = {}
        self.fn_filter_mode = "io"
        self.current_fn = None
        self._build_ui(use_flow)

    # ---- public API --------------------------------------------------------
    def setData(self, func_map: dict):
        """Provide/refresh the function index."""
        self.func_map = func_map or {}
        self._rebuild_fn_buttons(self.func_map.keys())

    def appendLog(self, text: str):
        """Append text to the console log."""
        cursor = self.log_view.textCursor()
        cursor.movePosition(QTextCursor.End)
        self.log_view.setTextCursor(cursor)
        self.log_view.insertPlainText(text)

    # ---- UI ---------------------------------------------------------------
    def _build_ui(self, use_flow: bool):
        root = QHBoxLayout(self)

        # left panel
        left = QVBoxLayout()
        row = QHBoxLayout()
        row.addWidget(QLabel("Scope:"))
        self.scope_combo = QComboBox(); self.scope_combo.addItems(["all", "reachable"])
        row.addWidget(self.scope_combo)
        left.addLayout(row)

        self.btn_scan = QPushButton("Scan Project Functions")
        left.addWidget(self.btn_scan)

        self.search_fn = QLineEdit(); self.search_fn.setPlaceholderText("Filter functions…")
        left.addWidget(self.search_fn)

        self.rb_fn_source = QRadioButton("Function")
        self.rb_fn_io = QRadioButton("Import/Export"); self.rb_fn_io.setChecked(True)
        self.rb_fn_all = QRadioButton("All")
        self.fn_filter_group = QButtonGroup(self)
        for rb in (self.rb_fn_source, self.rb_fn_io, self.rb_fn_all):
            self.fn_filter_group.addButton(rb); left.addWidget(rb)

        # scroll area for function "chips"
        self.fn_scroll = QScrollArea(); self.fn_scroll.setWidgetResizable(True)
        self.fn_container = QWidget()
        if use_flow:
            self.fn_layout = FlowLayout(self.fn_container, hspacing=8, vspacing=6)
            self.fn_container.setLayout(self.fn_layout)
        else:
            # fallback: vertical list aligned left
            box = QVBoxLayout(self.fn_container); box.setAlignment(Qt.AlignTop | Qt.AlignLeft)
            self.fn_layout = box
        self.fn_scroll.setWidget(self.fn_container)
        left.addWidget(self.fn_scroll)

        # right panel
        right = QVBoxLayout()
        right.addWidget(QLabel("Exported In"))
        self.exporters_list = QListWidget(); right.addWidget(self.exporters_list)
        right.addWidget(QLabel("Imported In"))
        self.importers_list = QListWidget(); right.addWidget(self.importers_list)
        right.addWidget(QLabel("Log"))
        self.log_view = QTextEdit(); self.log_view.setReadOnly(True); right.addWidget(self.log_view)

        root.addLayout(left, 1)
        root.addLayout(right, 2)

        # wire signals
        self.btn_scan.clicked.connect(lambda: self.scanRequested.emit(self.scope_combo.currentText()))
        self.search_fn.textChanged.connect(self._filter_fn_buttons)
        self.rb_fn_source.toggled.connect(lambda _: self._on_filter_mode_changed())
        self.rb_fn_io.toggled.connect(lambda _: self._on_filter_mode_changed())
        self.rb_fn_all.toggled.connect(lambda _: self._on_filter_mode_changed())
        # double-click to open in VS Code (optional)
        self.exporters_list.itemDoubleClicked.connect(lambda it: os.system(f'code -g "{it.text()}"'))
        self.importers_list.itemDoubleClicked.connect(lambda it: os.system(f'code -g "{it.text()}"'))

    # ---- internals --------------------------------------------------------
    def _add_fn_button(self, name: str):
        btn = QPushButton(name)
        btn.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        btn.clicked.connect(lambda _, n=name: self._on_function_clicked(n))
        # FlowLayout supports addWidget; QVBoxLayout does too
        self.fn_layout.addWidget(btn)

    def _clear_fn_buttons(self):
        while self.fn_layout.count():
            it = self.fn_layout.takeAt(0)
            w = it.widget()
            if w: w.deleteLater()

    def _rebuild_fn_buttons(self, names_iterable):
        self._clear_fn_buttons()
        names = sorted(n for n in names_iterable if n and n != '<reexport>')
        for name in names:
            self._add_fn_button(name)

    def _filter_fn_buttons(self, text: str):
        t = (text or '').strip().lower()
        if not self.func_map:
            return
        if not t:
            self._rebuild_fn_buttons(self.func_map.keys())
        else:
            match = [n for n in self.func_map.keys() if t in n.lower()]
            self._rebuild_fn_buttons(match)

    def _on_filter_mode_changed(self):
        self.fn_filter_mode = "source" if self.rb_fn_source.isChecked() else ("all" if self.rb_fn_all.isChecked() else "io")
        if self.current_fn:
            self._render_fn_lists_for(self.current_fn)

    def _on_function_clicked(self, fn_name: str):
        self.current_fn = fn_name
        self.functionSelected.emit(fn_name)
        self._render_fn_lists_for(fn_name)

    def _render_fn_lists_for(self, fn_name: str):
        self.exporters_list.clear()
        self.importers_list.clear()
        data = self.func_map.get(fn_name, {'exported_in': [], 'imported_in': []})

        exported_in, imported_in = [], []
        if isinstance(data, dict):
            exported_in = list(dict.fromkeys(data.get('exported_in', [])))
            imported_in = list(dict.fromkeys(data.get('imported_in', [])))
        elif isinstance(data, list):
            for d in data:
                if isinstance(d, dict):
                    exported_in += d.get('exported_in', [])
                    imported_in += d.get('imported_in', [])
                elif isinstance(d, str):
                    exported_in.append(d); imported_in.append(d)
            exported_in = list(dict.fromkeys(exported_in))
            imported_in = list(dict.fromkeys(imported_in))

        mode = self.fn_filter_mode
        if mode == "source":
            for f in sorted(exported_in): self.exporters_list.addItem(f)
        elif mode == "io":
            for f in sorted(exported_in): self.exporters_list.addItem(f)
            for f in sorted(imported_in): self.importers_list.addItem(f)
        else:  # all
            union = sorted(set(exported_in) | set(imported_in))
            for f in union: self.exporters_list.addItem(f)





#!/usr/bin/env python3
import os
import sys
import traceback
from typing import *
from dataclasses import dataclass
# e.g., if finder code is at project_root/tools/find_tools.py
sys.path.append("/path/to/project_root")
from abstract_paths import findContent, findContentAndEdit
from abstract_paths import get_directory_map, findGlobFiles,collect_filepaths,define_defaults,get_py_script_paths
from PyQt5.QtCore import Qt, QThread, pyqtSignal
from PyQt5.QtWidgets import (
    QApplication, QWidget, QVBoxLayout, QHBoxLayout, QGridLayout, QTabWidget,
    QLabel, QLineEdit, QPushButton, QTextEdit, QListWidget, QListWidgetItem,
    QCheckBox, QFileDialog, QSpinBox, QMessageBox
)
# your code: the functions you pasted
from abstract_paths import ( # <- change to your package path
    findContent, findContentAndEdit
)




from ..functions import *
from .initFuncs import initFuncs
class finderTab(QWidget):
    def __init__(self):
        super().__init__()
        self.setLayout(QVBoxLayout())
        grid = QGridLayout()
        # Directory picker
        self.dir_in = QLineEdit(os.getcwd())
        btn_browse = QPushButton("Browse…")
        btn_browse.clicked.connect(self.browse_dir)
        # Strings to find (comma-separated)
        self.strings_in = QLineEdit("")
        self.strings_in.setPlaceholderText("comma,separated,strings")
        # Allowed Extensions (comma-separated or pipe: ts,tsx,js,jsx or ts|tsx|js|jsx)
        self.allowed_exts_in = QLineEdit("ts,tsx,js,jsx,css")
        self.allowed_exts_in.setPlaceholderText("ts,tsx,js,jsx,css")
        # Unallowed Extensions (comma-separated or pipe)
        self.unallowed_exts_in = QLineEdit("")
        self.unallowed_exts_in.setPlaceholderText("comma,separated,unallowed,exts")
        # Exclude Types (comma-separated)
        self.exclude_types_in = QLineEdit("")
        self.exclude_types_in.setPlaceholderText("file,dir,etc")
        # Exclude Dirs (comma-separated)
        self.exclude_dirs_in = QLineEdit("")
        self.exclude_dirs_in.setPlaceholderText("dir1,dir2")
        # Exclude Patterns (comma-separated regex)
        self.exclude_patterns_in = QLineEdit("")
        self.exclude_patterns_in.setPlaceholderText("^src/,pattern2")
        # Add checkbox
        self.chk_add = QCheckBox("Add"); self.chk_add.setChecked(False)
        # Flags
        self.chk_recursive = QCheckBox("Recursive"); self.chk_recursive.setChecked(True)
        self.chk_total = QCheckBox("Require ALL strings (total_strings)"); self.chk_total.setChecked(False)
        self.chk_parse = QCheckBox("parse_lines"); self.chk_parse.setChecked(False)
        self.chk_getlines = QCheckBox("get_lines"); self.chk_getlines.setChecked(True)
        # Spec line (0 = off). Your code uses False to mean “disabled”, so we map 0→False.
        self.spec_spin = QSpinBox(); self.spec_spin.setRange(0, 999999)
        self.spec_spin.setValue(0)
        self.spec_spin.setToolTip("0 disables spec_line. >0 checks only that line (1-based).")
        # Run + Open in editor
        self.btn_run = QPushButton("Run search")
        self.btn_run.clicked.connect(self.start_search)
        self.btn_open_all = QPushButton("Open all hits in VS Code")
        self.btn_open_all.clicked.connect(self.open_all_hits)
        self.btn_open_all.setEnabled(False)
        # Layout form
        r = 0
        grid.addWidget(QLabel("Directory"), r, 0); grid.addWidget(self.dir_in, r, 1); grid.addWidget(btn_browse, r, 2); r+=1
        grid.addWidget(QLabel("Strings"), r, 0); grid.addWidget(self.strings_in, r, 1, 1, 2); r+=1
        grid.addWidget(QLabel("Allowed Exts"),r, 0); grid.addWidget(self.allowed_exts_in, r, 1, 1, 2); r+=1
        grid.addWidget(QLabel("Unallowed Exts"),r, 0); grid.addWidget(self.unallowed_exts_in, r, 1, 1, 2); r+=1
        grid.addWidget(QLabel("Exclude Types"),r, 0); grid.addWidget(self.exclude_types_in, r, 1, 1, 2); r+=1
        grid.addWidget(QLabel("Exclude Dirs"),r, 0); grid.addWidget(self.exclude_dirs_in, r, 1, 1, 2); r+=1
        grid.addWidget(QLabel("Exclude Patterns"),r, 0); grid.addWidget(self.exclude_patterns_in, r, 1, 1, 2); r+=1
        flag_row = QHBoxLayout()
        for w in (self.chk_recursive, self.chk_total, self.chk_parse, self.chk_getlines, self.chk_add):
            flag_row.addWidget(w)
        grid.addLayout(flag_row, r, 0, 1, 3); r+=1
        sp = QHBoxLayout()
        sp.addWidget(QLabel("spec_line (0=off):"))
        sp.addWidget(self.spec_spin)
        sp.addStretch(1)
        grid.addLayout(sp, r, 0, 1, 3); r+=1
        self.layout().addLayout(grid)
        cta = QHBoxLayout()
        cta.addWidget(self.btn_run)
        cta.addStretch(1)
        cta.addWidget(self.btn_open_all)
        self.layout().addLayout(cta)
        # Output area
        self.log = QTextEdit()
        self.log.setReadOnly(True)
        self.log.setLineWrapMode(QTextEdit.NoWrap)
        self.layout().addWidget(QLabel("Results"))
        self.layout().addWidget(self.log, stretch=2)
        self.list = QListWidget()
        self.list.itemDoubleClicked.connect(self.open_one)
        self.layout().addWidget(self.list, stretch=3)
        self._last_results = []
finderTab = initFuncs(finderTab)
